package ServerConfig;

import java.util.Properties;

import ServerConfig.loader.ProgramLoader;
import ServerConfig.loader.ProgramLoaderHelper;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CORBA.Policy;
import org.omg.PortableServer.POAPackage.*;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.*;

/**
 * @author Min Huang (khtwo)
 * Start up the loader servant as a persistant server and ready to receive request
 *
 */
public class LoaderServer {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("Loader Server start up.");
		ServerConfig currentConfig = new ServerConfig();
		currentConfig.centralNSConfigFileName = args[0] + "\\" + currentConfig.centralNSConfigFileName;
		currentConfig.serverConfigFileName = args[0] + "\\" + currentConfig.serverConfigFileName;
		currentConfig.monitorConfigFileName = args[0] + "\\" + currentConfig.monitorConfigFileName;
		currentConfig.currentDirectory = args[0];
		try {
			int stage = 0; // Config reading stage, 0: Totally failed, 1: Get Server cfg, 2: Get Monitor cfg
			// Try to read server config
			try{
				currentConfig.readConfigFromFile(true);
				stage = 1;				
			}catch(Exception e){
				// Read server config failed, try to read monitor config
				try{
					currentConfig.readConfigFromFile(false);
					stage = 2;				
				}catch(Exception f){
					// Read mointor config failed, do nothing and quit
					Runtime.getRuntime().exit(1);					
				}
			}
			
			// Install this loader in local orbd
			
			// Instantiate the local ORB
			Properties props = new Properties();
			props.put("org.omg.CORBA.ORBInitialPort", currentConfig.localNCInitialPort);
			props.put("org.omg.CORBA.ORBInitialHost", currentConfig.localhostIP);
			String empty[] = new String[0];
			ORB orb = ORB.init(empty, props);
			// Instantiate the servant
			LoaderServant loaderServant = new LoaderServant(orb);
			// Create a POA with Persistent Policy
			POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
			POA persistentPOA;
			try {
				persistentPOA = rootPOA.find_POA("PersistantPOA", true);
				System.out.println("Persistant POA found.");
			}
			catch (AdapterNonExistent ex) {
				// The specified POA does not exist, create it
				Policy[] persistentPolicy = new Policy[1];
				persistentPolicy[0] = rootPOA.create_lifespan_policy(LifespanPolicyValue.PERSISTENT);
				persistentPOA = rootPOA.create_POA("PersistantPOA", null, persistentPolicy);
				persistentPOA.the_POAManager().activate();
				System.out.println("Persistant POA created.");
			}
			// Associate the servant with PersistentPOA
			persistentPOA.activate_object(loaderServant);

			org.omg.CORBA.Object ref = persistentPOA.servant_to_reference(loaderServant);
			ProgramLoader href1 = ProgramLoaderHelper.narrow(ref);

			// Create naming context on central Naming service
			
			String corbaname = "corbaname::" + ServerConfig.centralNCInitialHost + ":" + ServerConfig.centralNCPort + "#";
			NamingContextExt ctx = NamingContextExtHelper.narrow(orb.string_to_object(corbaname));
			
			currentConfig.loader = href1;

			if(stage==1){
				// Current configure is for the server, register it on central naming server
				try{
					currentConfig.setupServer(ctx);
				}catch(Exception e){
					System.out.println("Exception while setting up the server: " + e);
				}

				// Read the configure for the monitor, register it on central naming service
				try{
					currentConfig.readConfigFromFile(false);
					currentConfig.loader = href1;
					stage = 2;
					// Register the monitor on central naming service
					try{
						currentConfig.setupMonitor(ctx);
					}catch(Exception e){
						System.out.println("Exception while setting up the monitor: " + e);
					}
				}catch(Exception f){
					// Read mointor config failed, do nothing
				}
				
			}else if(stage==2){
				// Current configure is for the monitor, register it on central naming server
				try{
					currentConfig.setupMonitor(ctx);
				}catch(Exception e){
					System.out.println("Exception while setting up the monitor: " + e);
				}
			}

			
			
			
/*  //Old register code, depreciated			
			
			// Add "Branches" naming context directory
			NameComponent name[] = ctx.to_name("Branches");
			NamingContext ctx2 = ctx;
			try {
				ctx2 = ctx.bind_new_context(name);
			} catch (Exception er) {
				if (er instanceof AlreadyBound) {
					// Already bound, get the context directly
					System.out.println("Branches is already bound");
					ctx2 = NamingContextExtHelper.narrow(ctx
							.resolve_str("Branches"));
				}
			}

			// Add certain "BranchXXX" naming context directory
			name = ctx.to_name("Branch"+currentConfig.branchID);
			ctx2 = ctx;
			try {
				ctx2 = ctx.bind_new_context(name);
			} catch (Exception er) {
				if (er instanceof AlreadyBound) {
					// Already bound, get the context directly
					System.out.println("Branch"+currentConfig.branchID + " is already bound");
					ctx2 = NamingContextExtHelper.narrow(ctx
							.resolve_str("Branch"+currentConfig.branchID));
				}
			}

			// Add a new directory under Branches directory
			name = ctx.to_name("Server"+currentConfig.serverID);
			try {
				ctx2 = ctx2.bind_new_context(name);
			} catch (Exception er) {
				if (er instanceof AlreadyBound) {
					// Already bound, get the context directly
					System.out.println("Server"+currentConfig.serverID+" is already bound");
					ctx2 = NamingContextExtHelper.narrow(((NamingContextExt) ctx2)
									.resolve_str("Server"+currentConfig.serverID));
				}
			}

			// Add text under directory BankServers\Server1
			name = ctx.to_name(currentConfig.getLoaderNC());
			ctx2.rebind(name, ctx2);
			
			NameComponent[] nc1 = ctx.to_name("loader" + currentConfig.branchID + "-" + currentConfig.serverID);
			href1 = ProgramLoaderHelper.narrow(ref);
			ctx2.rebind(nc1, href1);
*/
			

			orb.run();

		}
		catch (Exception e) {
			System.err.println("Exception in Persistent Server Startup " + e);
		}

	}

}
