
package BankApp;

import java.io.*;
import java.nio.*;
import java.util.*;
import java.net.*;




//let's use a redo logging algo, that is every operation is treated as a transaction
//the first thing to do is to write into log such that a transaction begins
//at the end when update is done to disk, then write into log that a transaction committed or roll-backed
public class LogManager
{
	protected static final long LogStartOffset=8*2;
	protected long packetNumber = 0;
	protected long checkPointer = 0;//the number of log should be started to check
	protected long currentPos = 0;
	protected long checkPos = 0;
	protected RandomAccessFile log;
	protected PacketWrapper wrapper;

	public LogManager(String logFileName)
	{
		try
		{
			//log=new RandomAccessFile(branchID+".log", "rw");
			log=new RandomAccessFile(logFileName, "rw");
			if (log.length()==0)
			{
				log.seek(0);			
				packetNumber=0;
				checkPointer=0;
				log.writeLong(packetNumber);
				log.writeLong(checkPointer);
			}
			else
			{
				log.seek(0);
				packetNumber=log.readLong();
				checkPointer=log.readLong();
			}			
			wrapper=new PacketWrapper();
			currentPos=packetNumber;
			checkPos=checkPointer;		
		}
		catch (Exception er)
		{
			System.out.println(er.getMessage());
		}
	}

	//////////////////////////////////////////////////
	//do we need reset current position?
	public synchronized void setCheckPoint()
	{
		try
		{
			checkPointer=packetNumber;
			log.seek(0);
			log.writeLong(packetNumber);
			log.writeLong(checkPointer);
		}
		catch (Exception er)
		{
			System.out.println(er.getMessage());
		}
	}

	/**
	 * Set the new check point, normally do this after sending out a log to backup server
	 * @param newCheckPoint
	 */
	public synchronized void setCheckPoint(long newCheckPoint)
	{
		try
		{
			// Check if the new check point is in the range
			if(newCheckPoint<=packetNumber){
				checkPointer=newCheckPoint;
				log.seek(0);
				log.writeLong(packetNumber);
				log.writeLong(checkPointer);
			}
		}
		catch (Exception er)
		{
			System.out.println(er.getMessage());
		}
	}

	/////////////////////////////////////////////////////

	public synchronized Packet findRecord(String opID)
	{
		Packet result=null;
		try
		{
			// log.seek(LogStartOffset+checkPointer*BankUtility.PacketSize);
			// Check whole records
			log.seek(LogStartOffset);
			do
			{
				result=wrapper.readPacket(log);
			}
			while (result==null?false:result.opID.compareTo(opID)!=0);
		}
		catch(Exception er)
		{
			result=null;//not find
		}
		return result;
	}


	/**
	 * Get current position
	 * @return
	 */
	public synchronized long getCurrentPos(){
		return this.currentPos;
	}
	
	
	/**
	 * Set current position
	 * @return
	 */
	public synchronized void setCurrentPos(long newPos){
		this.currentPos = newPos;
	}
	
	/**
	 * Find the absolute position of a packet
	 * @param packet The packet to be search
	 * @return Return the index of the packet or -1 if not found
	 */
	public synchronized int findRecord(Packet packet)
	{
		int index = 0;
		Packet result;
		try
		{
			log.seek(LogStartOffset);
			do
			{
				result=wrapper.readPacket(log);
				index++;
			}
			while (result.opID.compareTo(packet.opID)!=0);
		}
		catch(Exception er)
		{
			index=-1;//not find
		}
		return index;
	}

	public synchronized Packet readPacket() throws java.rmi.RemoteException
	{
		Packet result=null;
		try
		{
			// Debug
			BankUtility.showDebugInfo(1,"Log: current reading position:" +currentPos + ", packetNumber:" + packetNumber + " ");
			
			if (currentPos<packetNumber)
			{
				log.seek(currentPos*BankUtility.PacketSize+LogStartOffset);
				result= wrapper.readPacket(log);
				currentPos++;
			}
			else
			{
				throw new BankException("read invalid log");
			}
		}
		catch (Exception er)
		{
			result = null;
			// System.out.println(er.getMessage());
		}
		return result;
	}
	public synchronized void writePacket(Packet packet)
	{		
		try
		{
			// Check if the packet has been recorded
			if(findRecord(packet.opID)==null){
				log.seek(packetNumber*BankUtility.PacketSize+LogStartOffset);
				wrapper.writePacket(log, packet);
				packetNumber++;
				currentPos=packetNumber;
				log.seek(0);
				log.writeLong(packetNumber);
			}
		}
		catch (Exception er)
		{
			System.out.println(er.getMessage());
		}
	}

	//negative means going back to number of packet
	public synchronized void skipPacket(int number) throws java.rmi.RemoteException
	{
		long temp=currentPos+number;
		try
		{
			if (temp>=0&& temp<=packetNumber)
			{
				currentPos+=number;	
				log.seek(currentPos*BankUtility.PacketSize+LogStartOffset);		
			}
			else
			{
				throw new BankException("moving to invalid log position");	
			}		
		}
		catch (Exception er)
		{
			System.out.println(er.getMessage());
		}
	}
	public synchronized void skipToBeginning()
	{
		try
		{
			currentPos=0;
			log.seek(LogStartOffset);
		}
		catch (Exception er)
		{
			System.out.println(er.getMessage());
		}
	}

	public synchronized void gotoCheckPoint()
	{
		try
		{
			// Debug
			BankUtility.showDebugInfo(1, "Current checkpointer = " + checkPointer + ", packetNumber = " + packetNumber);

			// Set the position in file
			log.seek(checkPointer*BankUtility.PacketSize+LogStartOffset);
			// Set the current position
			this.currentPos = checkPointer;
		}
		catch (Exception er)
		{
			System.out.println(er.getMessage());
		}
	}
}
//Mar 21 ends